@@ -1,11 +1,16 @@
-// Copyright 2017 The Chromium Authors. All rights reserved.
+// Copyright 2017 The Chromium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include "cc/animation/worklet_animation.h"
 
+#include <memory>
 #include <utility>
+#include <vector>
 #include "base/memory/ptr_util.h"
+#include "base/time/time.h"
+#include "cc/animation/animation_id_provider.h"
+#include "cc/animation/keyframe_effect.h"
 #include "cc/animation/scroll_timeline.h"
 #include "cc/test/animation_test_common.h"
 #include "cc/test/animation_timelines_test_common.h"
@@ -25,8 +30,9 @@ namespace {
 
 class MockKeyframeEffect : public KeyframeEffect {
  public:
-  MockKeyframeEffect() : KeyframeEffect(0) {}
-  MOCK_METHOD1(Tick, void(base::TimeTicks monotonic_time));
+  explicit MockKeyframeEffect(Animation* animation)
+      : KeyframeEffect(animation) {}
+  MOCK_METHOD1(Tick, bool(base::TimeTicks monotonic_time));
 };
 
 class WorkletAnimationTest : public AnimationTimelinesTest {
@@ -39,7 +45,7 @@ class WorkletAnimationTest : public AnimationTimelinesTest {
 
     worklet_animation_ = WrapRefCounted(
         new WorkletAnimation(1, worklet_animation_id_, "test_name", 1, nullptr,
-                             nullptr, nullptr, true /* controlling instance*/));
+                             nullptr, true /* controlling instance*/));
     worklet_animation_->AttachElement(element_id_);
     host_->AddAnimationTimeline(timeline_);
     timeline_->AttachAnimation(worklet_animation_);
@@ -54,29 +60,29 @@ class MockScrollTimeline : public ScrollTimeline {
   MockScrollTimeline()
       : ScrollTimeline(ElementId(),
                        ScrollTimeline::ScrollDown,
-                       base::nullopt,
-                       base::nullopt,
-                       0,
-                       KeyframeModel::FillMode::NONE) {}
+                       /* scroll_offsets */ absl::nullopt,
+                       AnimationIdProvider::NextTimelineId()) {}
   MOCK_CONST_METHOD2(CurrentTime,
-                     base::Optional<base::TimeTicks>(const ScrollTree&, bool));
+                     absl::optional<base::TimeTicks>(const ScrollTree&, bool));
   MOCK_CONST_METHOD2(IsActive, bool(const ScrollTree&, bool));
+
+ protected:
+  ~MockScrollTimeline() override = default;
 };
 
 TEST_F(WorkletAnimationTest, NonImplInstanceDoesNotTickKeyframe) {
+  scoped_refptr<WorkletAnimation> worklet_animation = WrapRefCounted(
+      new WorkletAnimation(1, worklet_animation_id_, "test_name", 1, nullptr,
+                           nullptr, false /* not impl instance*/));
   std::unique_ptr<MockKeyframeEffect> effect =
-      std::make_unique<MockKeyframeEffect>();
+      std::make_unique<MockKeyframeEffect>(worklet_animation.get());
   MockKeyframeEffect* mock_effect = effect.get();
-
-  scoped_refptr<WorkletAnimation> worklet_animation =
-      WrapRefCounted(new WorkletAnimation(
-          1, worklet_animation_id_, "test_name", 1, nullptr, nullptr, nullptr,
-          false /* not impl instance*/, std::move(effect)));
+  worklet_animation->SetKeyframeEffectForTesting(std::move(effect));
 
   EXPECT_CALL(*mock_effect, Tick(_)).Times(0);
 
   MutatorOutputState::AnimationState state(worklet_animation_id_);
-  state.local_times.push_back(base::TimeDelta::FromSecondsD(1));
+  state.local_times.push_back(base::Seconds(1));
   worklet_animation->SetOutputState(state);
   worklet_animation->Tick(base::TimeTicks());
 }
@@ -98,7 +104,7 @@ TEST_F(WorkletAnimationTest, LocalTimeIsUsedWhenTicking) {
   // Impl side animation don't need synchronized start time.
   keyframe_model->set_needs_synchronized_start_time(false);
 
-  base::TimeDelta local_time = base::TimeDelta::FromSecondsD(duration / 2);
+  base::TimeDelta local_time = base::Seconds(duration / 2);
   MutatorOutputState::AnimationState state(worklet_animation_id_);
   state.local_times.push_back(local_time);
   worklet_animation_->SetOutputState(state);
@@ -116,7 +122,7 @@ TEST_F(WorkletAnimationTest, LocalTimeIsUsedWhenTicking) {
 TEST_F(WorkletAnimationTest, AnimationEventLocalTimeUpdate) {
   AttachWorkletAnimation();
 
-  base::Optional<base::TimeDelta> local_time = base::TimeDelta::FromSecondsD(1);
+  absl::optional<base::TimeDelta> local_time = base::Seconds(1);
   MutatorOutputState::AnimationState state(worklet_animation_id_);
   state.local_times.push_back(local_time);
   worklet_animation_->SetOutputState(state);
@@ -126,6 +132,8 @@ TEST_F(WorkletAnimationTest, AnimationEventLocalTimeUpdate) {
   worklet_animation_->UpdateState(true, animation_events);
 
   // One event is generated as a result of update state.
+  EXPECT_TRUE(animation_events->needs_time_updated_events());
+  worklet_animation_->TakeTimeUpdatedEvent(animation_events);
   EXPECT_EQ(1u, animation_events->events_.size());
   AnimationEvent event = animation_events->events_[0];
   EXPECT_EQ(AnimationEvent::TIME_UPDATED, event.type);
@@ -136,37 +144,39 @@ TEST_F(WorkletAnimationTest, AnimationEventLocalTimeUpdate) {
   mutator_events = host_->CreateEvents();
   animation_events = static_cast<AnimationEvents*>(mutator_events.get());
   worklet_animation_->UpdateState(true, animation_events);
-  EXPECT_EQ(0u, animation_events->events_.size());
+  EXPECT_FALSE(animation_events->needs_time_updated_events());
 
   // If local time is set to the same value no event is generated.
   worklet_animation_->SetOutputState(state);
   mutator_events = host_->CreateEvents();
   animation_events = static_cast<AnimationEvents*>(mutator_events.get());
   worklet_animation_->UpdateState(true, animation_events);
-  EXPECT_EQ(0u, animation_events->events_.size());
+  EXPECT_FALSE(animation_events->needs_time_updated_events());
 
   // If local time is set to null value, an animation event with null local
   // time is generated.
   state.local_times.clear();
-  local_time = base::nullopt;
+  local_time = absl::nullopt;
   state.local_times.push_back(local_time);
   worklet_animation_->SetOutputState(state);
   mutator_events = host_->CreateEvents();
   animation_events = static_cast<AnimationEvents*>(mutator_events.get());
   worklet_animation_->UpdateState(true, animation_events);
+  EXPECT_TRUE(animation_events->needs_time_updated_events());
+  worklet_animation_->TakeTimeUpdatedEvent(animation_events);
   EXPECT_EQ(1u, animation_events->events_.size());
   EXPECT_EQ(local_time, animation_events->events_[0].local_time);
 }
 
 TEST_F(WorkletAnimationTest, CurrentTimeCorrectlyUsesScrollTimeline) {
-  auto scroll_timeline = std::make_unique<MockScrollTimeline>();
+  auto scroll_timeline = base::WrapRefCounted(new MockScrollTimeline());
   EXPECT_CALL(*scroll_timeline, IsActive(_, _)).WillRepeatedly(Return(true));
   EXPECT_CALL(*scroll_timeline, CurrentTime(_, _))
-      .WillRepeatedly(Return(
-          (base::TimeTicks() + base::TimeDelta::FromMilliseconds(1234))));
-  scoped_refptr<WorkletAnimation> worklet_animation =
-      WorkletAnimation::Create(worklet_animation_id_, "test_name", 1,
-                               std::move(scroll_timeline), nullptr, nullptr);
+      .WillRepeatedly(Return((base::TimeTicks() + base::Milliseconds(1234))));
+  scoped_refptr<WorkletAnimation> worklet_animation = WorkletAnimation::Create(
+      worklet_animation_id_, "test_name", 1, nullptr, nullptr);
+  host_->AddAnimationTimeline(scroll_timeline);
+  scroll_timeline->AttachAnimation(worklet_animation);
 
   ScrollTree scroll_tree;
   std::unique_ptr<MutatorInputState> state =
@@ -181,14 +191,17 @@ TEST_F(WorkletAnimationTest, CurrentTimeCorrectlyUsesScrollTimeline) {
 TEST_F(WorkletAnimationTest,
        CurrentTimeFromRegularTimelineIsOffsetByStartTime) {
   scoped_refptr<WorkletAnimation> worklet_animation = WorkletAnimation::Create(
-      worklet_animation_id_, "test_name", 1, nullptr, nullptr, nullptr);
+      worklet_animation_id_, "test_name", 1, nullptr, nullptr);
 
-  base::TimeTicks first_ticks =
-      base::TimeTicks() + base::TimeDelta::FromMillisecondsD(111);
+  worklet_animation->AttachElement(element_id_);
+  host_->AddAnimationTimeline(timeline_);
+  timeline_->AttachAnimation(worklet_animation);
+
+  base::TimeTicks first_ticks = base::TimeTicks() + base::Milliseconds(111);
   base::TimeTicks second_ticks =
-      base::TimeTicks() + base::TimeDelta::FromMillisecondsD(111 + 123.4);
+      base::TimeTicks() + base::Milliseconds(111 + 123.4);
   base::TimeTicks third_ticks =
-      base::TimeTicks() + base::TimeDelta::FromMillisecondsD(111 + 246.8);
+      base::TimeTicks() + base::Milliseconds(111 + 246.8);
 
   ScrollTree scroll_tree;
   std::unique_ptr<MutatorInputState> state =
@@ -199,13 +212,13 @@ TEST_F(WorkletAnimationTest,
   std::unique_ptr<AnimationWorkletInput> input =
       state->TakeWorkletState(worklet_animation_id_.worklet_id);
   EXPECT_EQ(0, input->added_and_updated_animations[0].current_time);
-  state.reset(new MutatorInputState);
+  state = std::make_unique<MutatorInputState>();
   worklet_animation->UpdateInputState(state.get(), second_ticks, scroll_tree,
                                       true);
   input = state->TakeWorkletState(worklet_animation_id_.worklet_id);
   EXPECT_EQ(123.4, input->updated_animations[0].current_time);
   // Should always offset from start time.
-  state.reset(new MutatorInputState());
+  state = std::make_unique<MutatorInputState>();
   worklet_animation->UpdateInputState(state.get(), third_ticks, scroll_tree,
                                       true);
   input = state->TakeWorkletState(worklet_animation_id_.worklet_id);
@@ -219,15 +232,17 @@ TEST_F(WorkletAnimationTest, DocumentTimelineSetPlaybackRate) {
   const double playback_rate_half = 0.5;
   scoped_refptr<WorkletAnimation> worklet_animation =
       WorkletAnimation::Create(worklet_animation_id_, "test_name",
-                               playback_rate_double, nullptr, nullptr, nullptr);
+                               playback_rate_double, nullptr, nullptr);
 
-  base::TimeTicks first_ticks =
-      base::TimeTicks() + base::TimeDelta::FromMillisecondsD(111);
+  worklet_animation->AttachElement(element_id_);
+  host_->AddAnimationTimeline(timeline_);
+  timeline_->AttachAnimation(worklet_animation);
+
+  base::TimeTicks first_ticks = base::TimeTicks() + base::Milliseconds(111);
   base::TimeTicks second_ticks =
-      base::TimeTicks() + base::TimeDelta::FromMillisecondsD(111 + 123.4);
+      base::TimeTicks() + base::Milliseconds(111 + 123.4);
   base::TimeTicks third_ticks =
-      base::TimeTicks() +
-      base::TimeDelta::FromMillisecondsD(111 + 123.4 + 200.0);
+      base::TimeTicks() + base::Milliseconds(111 + 123.4 + 200.0);
 
   ScrollTree scroll_tree;
   std::unique_ptr<MutatorInputState> state =
@@ -235,7 +250,7 @@ TEST_F(WorkletAnimationTest, DocumentTimelineSetPlaybackRate) {
   // Start the animation.
   worklet_animation->UpdateInputState(state.get(), first_ticks, scroll_tree,
                                       true);
-  state.reset(new MutatorInputState);
+  state = std::make_unique<MutatorInputState>();
 
   // Play until second_ticks.
   worklet_animation->UpdateInputState(state.get(), second_ticks, scroll_tree,
@@ -250,7 +265,7 @@ TEST_F(WorkletAnimationTest, DocumentTimelineSetPlaybackRate) {
 
   // Update the playback rate.
   worklet_animation->SetPlaybackRateForTesting(playback_rate_half);
-  state.reset(new MutatorInputState());
+  state = std::make_unique<MutatorInputState>();
 
   // Play until third_ticks.
   worklet_animation->UpdateInputState(state.get(), third_ticks, scroll_tree,
@@ -267,14 +282,16 @@ TEST_F(WorkletAnimationTest, DocumentTimelineSetPlaybackRate) {
 TEST_F(WorkletAnimationTest, ScrollTimelineSetPlaybackRate) {
   const double playback_rate_double = 2;
   const double playback_rate_half = 0.5;
-  auto scroll_timeline = std::make_unique<MockScrollTimeline>();
+  auto scroll_timeline = base::WrapRefCounted(new MockScrollTimeline());
 
-  scoped_refptr<WorkletAnimation> worklet_animation = WorkletAnimation::Create(
-      worklet_animation_id_, "test_name", playback_rate_double,
-      std::move(scroll_timeline), nullptr, nullptr);
+  scoped_refptr<WorkletAnimation> worklet_animation =
+      WorkletAnimation::Create(worklet_animation_id_, "test_name",
+                               playback_rate_double, nullptr, nullptr);
+  host_->AddAnimationTimeline(scroll_timeline);
+  scroll_timeline->AttachAnimation(worklet_animation);
   const MockScrollTimeline* mock_timeline =
       static_cast<const MockScrollTimeline*>(
-          worklet_animation->scroll_timeline());
+          worklet_animation->animation_timeline());
 
   ScrollTree scroll_tree;
   std::unique_ptr<MutatorInputState> state =
@@ -282,8 +299,7 @@ TEST_F(WorkletAnimationTest, ScrollTimelineSetPlaybackRate) {
   // Start the animation.
   EXPECT_CALL(*mock_timeline, IsActive(_, _)).WillRepeatedly(Return(true));
   EXPECT_CALL(*mock_timeline, CurrentTime(_, _))
-      .WillRepeatedly(
-          Return(base::TimeTicks() + base::TimeDelta::FromMilliseconds(50)));
+      .WillRepeatedly(Return(base::TimeTicks() + base::Milliseconds(50)));
   worklet_animation->UpdateInputState(state.get(), base::TimeTicks(),
                                       scroll_tree, true);
   Mock::VerifyAndClearExpectations(&mock_timeline);
@@ -297,13 +313,12 @@ TEST_F(WorkletAnimationTest, ScrollTimelineSetPlaybackRate) {
 
   // Update the playback rate.
   worklet_animation->SetPlaybackRateForTesting(playback_rate_half);
-  state.reset(new MutatorInputState());
+  state = std::make_unique<MutatorInputState>();
 
   // Continue playing the animation.
   EXPECT_CALL(*mock_timeline, IsActive(_, _)).WillRepeatedly(Return(true));
   EXPECT_CALL(*mock_timeline, CurrentTime(_, _))
-      .WillRepeatedly(
-          Return(base::TimeTicks() + base::TimeDelta::FromMilliseconds(100)));
+      .WillRepeatedly(Return(base::TimeTicks() + base::Milliseconds(100)));
   worklet_animation->UpdateInputState(state.get(), base::TimeTicks(),
                                       scroll_tree, true);
   Mock::VerifyAndClearExpectations(&mock_timeline);
@@ -317,15 +332,17 @@ TEST_F(WorkletAnimationTest, ScrollTimelineSetPlaybackRate) {
 // Verifies correcteness of worklet animation current time when inactive
 // timeline becomes active and then inactive again.
 TEST_F(WorkletAnimationTest, InactiveScrollTimeline) {
-  auto scroll_timeline = std::make_unique<MockScrollTimeline>();
+  auto scroll_timeline = base::WrapRefCounted(new MockScrollTimeline());
 
-  scoped_refptr<WorkletAnimation> worklet_animation = WorkletAnimation::Create(
-      worklet_animation_id_, "test_name", /*playback_rate*/ 1,
-      std::move(scroll_timeline), nullptr, nullptr);
+  scoped_refptr<WorkletAnimation> worklet_animation =
+      WorkletAnimation::Create(worklet_animation_id_, "test_name",
+                               /*playback_rate*/ 1, nullptr, nullptr);
 
+  host_->AddAnimationTimeline(scroll_timeline);
+  scroll_timeline->AttachAnimation(worklet_animation);
   const MockScrollTimeline* mock_timeline =
       static_cast<const MockScrollTimeline*>(
-          worklet_animation->scroll_timeline());
+          worklet_animation->animation_timeline());
   ScrollTree scroll_tree;
   std::unique_ptr<MutatorInputState> state =
       std::make_unique<MutatorInputState>();
@@ -338,13 +355,12 @@ TEST_F(WorkletAnimationTest, InactiveScrollTimeline) {
   std::unique_ptr<AnimationWorkletInput> input =
       state->TakeWorkletState(worklet_animation_id_.worklet_id);
   EXPECT_FALSE(input);
-  state.reset(new MutatorInputState());
+  state = std::make_unique<MutatorInputState>();
 
   // Now the timeline is active.
   EXPECT_CALL(*mock_timeline, IsActive(_, _)).WillRepeatedly(Return(true));
   EXPECT_CALL(*mock_timeline, CurrentTime(_, _))
-      .WillRepeatedly(
-          Return(base::TimeTicks() + base::TimeDelta::FromMilliseconds(100)));
+      .WillRepeatedly(Return(base::TimeTicks() + base::Milliseconds(100)));
   worklet_animation->UpdateInputState(state.get(), base::TimeTicks(),
                                       scroll_tree, true);
   Mock::VerifyAndClearExpectations(&mock_timeline);
@@ -352,13 +368,12 @@ TEST_F(WorkletAnimationTest, InactiveScrollTimeline) {
   // Verify that the current time is updated when the timeline becomes newly
   // active.
   EXPECT_EQ(100, input->added_and_updated_animations[0].current_time);
-  state.reset(new MutatorInputState());
+  state = std::make_unique<MutatorInputState>();
 
   // Now the timeline is inactive.
   EXPECT_CALL(*mock_timeline, IsActive(_, _)).WillRepeatedly(Return(false));
   EXPECT_CALL(*mock_timeline, CurrentTime(_, _))
-      .WillRepeatedly(
-          Return(base::TimeTicks() + base::TimeDelta::FromMilliseconds(200)));
+      .WillRepeatedly(Return(base::TimeTicks() + base::Milliseconds(200)));
   worklet_animation->UpdateInputState(state.get(), base::TimeTicks(),
                                       scroll_tree, true);
   Mock::VerifyAndClearExpectations(&mock_timeline);
@@ -397,8 +412,8 @@ TEST_F(WorkletAnimationTest, UpdateInputStateProducesCorrectState) {
 
   // The state of WorkletAnimation is updated to RUNNING after calling
   // UpdateInputState above.
-  state.reset(new MutatorInputState());
-  time += base::TimeDelta::FromSecondsD(0.1);
+  state = std::make_unique<MutatorInputState>();
+  time += base::Seconds(0.1);
   worklet_animation_->UpdateInputState(state.get(), time, scroll_tree, true);
   input = state->TakeWorkletState(worklet_animation_id_.worklet_id);
   EXPECT_EQ(input->added_and_updated_animations.size(), 0u);
@@ -408,8 +423,8 @@ TEST_F(WorkletAnimationTest, UpdateInputStateProducesCorrectState) {
   // Operating on individual KeyframeModel doesn't affect the state of
   // WorkletAnimation.
   keyframe_model->SetRunState(KeyframeModel::FINISHED, time);
-  state.reset(new MutatorInputState());
-  time += base::TimeDelta::FromSecondsD(0.1);
+  state = std::make_unique<MutatorInputState>();
+  time += base::Seconds(0.1);
   worklet_animation_->UpdateInputState(state.get(), time, scroll_tree, true);
   input = state->TakeWorkletState(worklet_animation_id_.worklet_id);
   EXPECT_EQ(input->added_and_updated_animations.size(), 0u);
@@ -420,7 +435,7 @@ TEST_F(WorkletAnimationTest, UpdateInputStateProducesCorrectState) {
   // leads to RemoveKeyframeModel.
   worklet_animation_->RemoveKeyframeModel(keyframe_model_id);
   worklet_animation_->UpdateState(true, nullptr);
-  state.reset(new MutatorInputState());
+  state = std::make_unique<MutatorInputState>();
   worklet_animation_->UpdateInputState(state.get(), time, scroll_tree, true);
   input = state->TakeWorkletState(worklet_animation_id_.worklet_id);
   EXPECT_EQ(input->added_and_updated_animations.size(), 0u);
@@ -451,20 +466,20 @@ TEST_F(WorkletAnimationTest, SkipUnchangedAnimations) {
   EXPECT_EQ(input->added_and_updated_animations.size(), 1u);
   EXPECT_EQ(input->updated_animations.size(), 0u);
 
-  state.reset(new MutatorInputState());
+  state = std::make_unique<MutatorInputState>();
   // No update on the input state if input time stays the same.
   worklet_animation_->UpdateInputState(state.get(), time, scroll_tree, true);
   input = state->TakeWorkletState(worklet_animation_id_.worklet_id);
   EXPECT_FALSE(input);
 
-  state.reset(new MutatorInputState());
+  state = std::make_unique<MutatorInputState>();
   // Different input time causes the input state to be updated.
-  time += base::TimeDelta::FromSecondsD(0.1);
+  time += base::Seconds(0.1);
   worklet_animation_->UpdateInputState(state.get(), time, scroll_tree, true);
   input = state->TakeWorkletState(worklet_animation_id_.worklet_id);
   EXPECT_EQ(input->updated_animations.size(), 1u);
 
-  state.reset(new MutatorInputState());
+  state = std::make_unique<MutatorInputState>();
   // Input state gets updated when the worklet animation is to be removed even
   // the input time doesn't change.
   worklet_animation_->RemoveKeyframeModel(keyframe_model_id);
@@ -474,23 +489,24 @@ TEST_F(WorkletAnimationTest, SkipUnchangedAnimations) {
   EXPECT_EQ(input->removed_animations.size(), 1u);
 }
 
-base::Optional<base::TimeTicks> FakeIncreasingScrollTimelineTime(Unused,
+absl::optional<base::TimeTicks> FakeIncreasingScrollTimelineTime(Unused,
                                                                  Unused) {
   static base::TimeTicks current_time;
-  current_time += base::TimeDelta::FromSecondsD(0.1);
+  current_time += base::Seconds(0.1);
   return current_time;
 }
 
 // This test verifies that worklet animation gets skipped properly if a pending
 // mutation cycle is holding a lock on the worklet.
 TEST_F(WorkletAnimationTest, SkipLockedAnimations) {
-  auto scroll_timeline = std::make_unique<MockScrollTimeline>();
+  auto scroll_timeline = base::WrapRefCounted(new MockScrollTimeline());
   EXPECT_CALL(*scroll_timeline, IsActive(_, _)).WillRepeatedly(Return(true));
   EXPECT_CALL(*scroll_timeline, CurrentTime(_, _))
       .WillRepeatedly(Invoke(FakeIncreasingScrollTimelineTime));
-  scoped_refptr<WorkletAnimation> worklet_animation =
-      WorkletAnimation::Create(worklet_animation_id_, "test_name", 1,
-                               std::move(scroll_timeline), nullptr, nullptr);
+  scoped_refptr<WorkletAnimation> worklet_animation = WorkletAnimation::Create(
+      worklet_animation_id_, "test_name", 1, nullptr, nullptr);
+  host_->AddAnimationTimeline(scroll_timeline);
+  scroll_timeline->AttachAnimation(worklet_animation);
 
   ScrollTree scroll_tree;
   std::unique_ptr<MutatorInputState> state =
@@ -503,20 +519,20 @@ TEST_F(WorkletAnimationTest, SkipLockedAnimations) {
   EXPECT_EQ(input->added_and_updated_animations.size(), 1u);
   EXPECT_EQ(input->updated_animations.size(), 0u);
 
-  state.reset(new MutatorInputState());
+  state = std::make_unique<MutatorInputState>();
   // Different scroll time causes the input state to be updated.
   worklet_animation->UpdateInputState(state.get(), time, scroll_tree, true);
   input = state->TakeWorkletState(worklet_animation_id_.worklet_id);
   EXPECT_EQ(input->updated_animations.size(), 1u);
 
-  state.reset(new MutatorInputState());
+  state = std::make_unique<MutatorInputState>();
   // Different scroll time causes the input state to be updated. Pending
   // mutation will grab a lock.
   worklet_animation->UpdateInputState(state.get(), time, scroll_tree, false);
   input = state->TakeWorkletState(worklet_animation_id_.worklet_id);
   EXPECT_EQ(input->updated_animations.size(), 1u);
 
-  state.reset(new MutatorInputState());
+  state = std::make_unique<MutatorInputState>();
   // Pending lock has not been released.
   worklet_animation->UpdateInputState(state.get(), time, scroll_tree, true);
   input = state->TakeWorkletState(worklet_animation_id_.worklet_id);
@@ -524,7 +540,7 @@ TEST_F(WorkletAnimationTest, SkipLockedAnimations) {
 
   worklet_animation->ReleasePendingTreeLock();
 
-  state.reset(new MutatorInputState());
+  state = std::make_unique<MutatorInputState>();
   // Pending lock has been released.
   worklet_animation->UpdateInputState(state.get(), time, scroll_tree, true);
   input = state->TakeWorkletState(worklet_animation_id_.worklet_id);
